﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using Microsoft.Win32;
using System;
using System.Text;

namespace Check
{
    public class RegistryCheck
    {
        private const string RegistryFieldSubName = "Parameters";
        private const int NothingInRegistry = -1;

        private static bool m_FullVersion = true;
        private static int m_EvaluationDays = 14;
        private static string AppName;

        private const char FULL_VERSION = 'y';
        private const char EVALUATION_VERSION = 'o';

        private static string RegistryFieldName
        {
            get
            {
                return "Software\\" + AppName;
            }
        }

        private static string RegistryValue
        {
            get
            {
                try
                {
                    RegistryKey key = Registry.CurrentUser.OpenSubKey(RegistryFieldName);

                    return key.GetValue(RegistryFieldSubName) as string;
                }
                catch (Exception ex)
                {
                    return null;
                }
            }
            set
            {
                try
                {
                    RegistryKey key = Registry.CurrentUser.OpenSubKey(RegistryFieldName);

                    RegistryKey keyW = Registry.CurrentUser.CreateSubKey(RegistryFieldName);

                    keyW.SetValue(RegistryFieldSubName, value);
                }
                catch (Exception ex)
                {

                }
            }
        }

        private static string Encode(string val)
        {
            bool was_first = false;
            StringBuilder bld = new StringBuilder();

            foreach (char c in val)
            {
                if (c <= '9' && c >= '0')
                {
                    bld.Append((char)('a' + c - '0'));
                }
                else if (c == '.')
                {
                    if (!was_first)
                    {
                        was_first = true;
                        bld.Append('m');
                    }
                    else
                    {
                        bld.Append('n');
                    }
                }
                else if (c == FULL_VERSION || c == EVALUATION_VERSION)
                {
                    bld.Append(c);
                }
            }
            return bld.ToString();
        }

        private static string Decode(string val)
        {
            StringBuilder bld = new StringBuilder();

            foreach (char c in val)
            {
                if (c <= 'z' && c >= 'a' && c != 'm' && c != 'n' && c != 'y' && c != 'o')
                {
                    bld.Append((int)(c - 'a'));
                }
                else if (c == 'm' || c == 'n')
                {
                    bld.Append('.');
                }
                else if (c == FULL_VERSION || c == EVALUATION_VERSION)
                {
                    bld.Append(c);
                }
            }
            return bld.ToString();
        }

        private static int NumberOfDaysLeft(string val)
        {
            //mm.dd.yyyy
            val = Decode(val);
            string[] arr = val.Split('.');

            if (arr == null || arr.Length != 3)
                return 0;

            DateTime date = new DateTime(Int32.Parse(arr[2]), Int32.Parse(arr[0]), Int32.Parse(arr[1]));

            TimeSpan span = DateTime.Now - date;
            return m_EvaluationDays - span.Days;
        }

        private static int EvaluationDays
        {
            get
            {
                string regVal = RegistryValue;

                if (regVal == null)
                    return NothingInRegistry;
                else
                {
                    return NumberOfDaysLeft(regVal.Substring(1, regVal.Length - 1));
                }
            }
        }

        public static bool FullVersion
        {
            get
            {
                string str = RegistryValue;

                if (str == null)
                {
                    if (m_FullVersion)
                    {
                        string val = GetDateInString();

                        val = FULL_VERSION + val;
                        val = Encode(val);
                        RegistryValue = val;
                        return true;
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    str = Decode(str);

                    if (str[0] == FULL_VERSION)
                    {
                        return true;
                    }
                    else
                    {
                        if (m_FullVersion)
                        {
                            string str2 = FULL_VERSION + str.Substring(1, str.Length - 1);

                            str2 = Encode(str2);
                            RegistryValue = str2;
                            return true;
                        }
                        else
                        {
                            return false;
                        }
                    }
                }
            }
        }

        public static string ApplicationName
        {
            set
            {
                AppName = value;
            }
        }

        private static string GetDateInString()
        {
            DateTime now = DateTime.Now;

            string val = now.Month.ToString() + "." + now.Day.ToString() + "." + now.Year.ToString();

            return val;
        }

        public static bool EvaluationVersion(out string text)
        {
            int days = EvaluationDays;

            if (days == NothingInRegistry)
            {
                string val = EVALUATION_VERSION + GetDateInString();

                val = Encode(val);
                RegistryValue = val;
                days = m_EvaluationDays;
            }
            else if (days <= 0)
            {
                text = "Evaluation period has ended. Please buy full version.";
                return false;
            }

            text = String.Format("You now have {0} days left of Evaluation period.", days);
            return true;
        }
    }
}
