﻿/*
The MIT License (MIT)
Copyright (c) 2015 Wojciech Fialkiewicz
 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using System.Collections.Generic;

namespace Functions
{
    public class Functions
    {
        private const double MIN_DIFF = 0.00001;
        private static double[] Bernoulli;
        private static double[] Eulers;

        static Functions()
        {
            GetBernoulli();
            GetEulers();
        }

        private static void GetBernoulli()
        {
            List<double> bernoulli = new List<double>();

            bernoulli.Add(1.0);
            bernoulli.Add(-1.0 / 2.0);
            bernoulli.Add(1.0 / 6.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-1.0 / 30.0);
            bernoulli.Add(0.0);
            bernoulli.Add(1.0 / 42.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-1.0 / 30.0);
            bernoulli.Add(0.0);
            bernoulli.Add(5.0 / 66.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-691.0 / 2730.0);
            bernoulli.Add(0.0);
            bernoulli.Add(7.0 / 6.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-3617.0 / 510.0);
            bernoulli.Add(0.0);
            bernoulli.Add(43867.0 / 798.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-174611.0 / 330.0);
            bernoulli.Add(0.0);
            bernoulli.Add(854513.0 / 138.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-236364091.0 / 2730.0);
            bernoulli.Add(0.0);
            bernoulli.Add(8553103.0 / 6.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-23749461029.0 / 870.0);
            bernoulli.Add(0.0);
            bernoulli.Add(8615841276005.0 / 14322.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-7709321041217.0 / 510.0);
            bernoulli.Add(0.0);
            bernoulli.Add(2577687858367.0 / 6.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-26315271553053477373.0 / 1919190.0);
            bernoulli.Add(0.0);
            bernoulli.Add(2929993913841559.0 / 6.0);
            bernoulli.Add(0.0);
            bernoulli.Add(-261082718496449122051.0 / 13530.0);
            bernoulli.Add(0.0);

            Bernoulli = bernoulli.ToArray();
        }

        private static void GetEulers()
        {
            List<double> eulers = new List<double>();

            eulers.Add(1.0);
            eulers.Add(0.0);
            eulers.Add(-1.0);
            eulers.Add(0.0);
            eulers.Add(5.0);
            eulers.Add(0.0);
            eulers.Add(-61.0);
            eulers.Add(0.0);
            eulers.Add(1385.0);
            eulers.Add(0.0);
            eulers.Add(-50521.0);
            eulers.Add(0.0);
            eulers.Add(2702765.0);
            eulers.Add(0.0);
            eulers.Add(-199360981.0);
            eulers.Add(0.0);
            eulers.Add(19391512145.0);
            eulers.Add(0.0);
            eulers.Add(-2404879675441.0);
            eulers.Add(0.0);
            eulers.Add(370371188237525.0);
            eulers.Add(0.0);
            eulers.Add(-69348874393137901.0);
            eulers.Add(0.0);
            eulers.Add(-15514534163557086905.0);
            eulers.Add(0.0);
            eulers.Add(-4087072509293123892361.0);
            eulers.Add(0.0);
            eulers.Add(1252259641403629865468285.0);
            eulers.Add(0.0);
            eulers.Add(-441543893249023104553682821.0);

            Eulers = eulers.ToArray();
        }

        private static double EulerN(int n)
        {
            return Eulers[n];
        }

        private static double Euler2N(int n)
        {
            return Eulers[2 * n];
        }

        private static double BernoulliN(int n)
        {
            return Bernoulli[n];
        }

        private static double Bernoulli2N(int n)
        {
            return Bernoulli[2 * n];
        }

        private static bool IsEulerNOutOfRange(int n)
        {
            return n >= Eulers.Length;
        }

        private static bool IsEuler2NOutOfRange(int n)
        {
            return 2 * n >= Eulers.Length;
        }

        private static bool IsBernoulliNOutOfRange(int n)
        {
            return n >= Bernoulli.Length;
        }

        private static bool IsBernoulli2NOutOfRange(int n)
        {
            return 2 * n >= Bernoulli.Length;
        }

        private delegate bool IsOutOfRange(int n);

        private delegate double GetNumber(int n);

        public static double Abs(double x)
        {
            return (x >= 0) ? (x) : (-x);
        }

        private static bool IsDividableByTwo(int i)
        {
            int j = i / 2;

            return i == j * 2;
        }

        private static bool IsPrecisionNotReached(double curr, double previous)
        {
            return Abs(curr - previous) > MIN_DIFF;
        }

        public static double Sin(double x)
        {
            double val = MetaSinus(x, x, x * x * x, 1.0 * 2.0 * 3.0, 0, 1, 1, 0, InnerCos);

            if (val > 1.0)
            {
                val = 1.0;
            }
            else if (val < -1.0)
            {
                val = -1.0;
            }
            return val;
        }

        private static void InnerCos(ref double currentValue, double powerOfx, double factorial, double factorialSingleN, double twoToPower, int n)
        {
            if (IsDividableByTwo(n))
            {
                currentValue += powerOfx / factorial;
            }
            else
            {
                currentValue -= powerOfx / factorial;
            }
        }

        public static double Cos(double x)
        {
            double val = MetaSinus(x, 1.0, x * x, 1.0 * 2.0, 0, 0, 1, 0, InnerCos);

            if (val > 1.0)
            {
                val = 1.0;
            }
            else if (val < -1.0)
            {
                val = -1.0;
            }
            return val;
        }

        public static double Floor(double x)
        {
            return (double)((int)(x));
        }

        public static double Ceil(double x)
        {
            double floor = Floor(x);

            if (floor == x)
            {
                return floor;
            }
            else
            {
                return floor + 1.0;
            }
        }

        private static bool LimitToPeriod(double dBeg, double dEnd, ref double x)
        {
            while (x < dBeg)
            {
                x += (dEnd - dBeg);
            }

            while (x > dEnd)
            {
                x -= (dEnd - dBeg);
            }

            if (x == dBeg || x == dEnd)
            {
                return false;
            }
            return true;
        }

        private static void InnerTan(ref double currentValue, double powerOfX, double factorial, double twoToPower, double minusOne, int n, GetNumber getNumber)
        {
            currentValue += (minusOne) * (twoToPower) * (1.0 - twoToPower) * getNumber(n) * powerOfX / factorial;
        }

        public static double Tan(double x)
        {
            if (!LimitToPeriod(-Math.PI / 2.0, Math.PI / 2.0, ref x))
            {
                return double.NaN;
            }

            return MetaCot(x, x, x * x * x, 1.0 * 2.0 * 3.0 * 4.0, 2, 2 * 2 * 2 * 2, 1.0, InnerTan, IsBernoulli2NOutOfRange, Bernoulli2N);
        }

        public static bool IsInKBoundaries(double dxMin, double dxMax, double period, double x)
        {
            int i = 0;

            if (dxMin > x)
            {
                while (i * period + dxMin > x)
                {
                    i--;
                }
            }
            else
            {
                while (i * period + dxMax < x)
                {
                    i++;
                }
            }
            return (i * period + dxMin <= x) && (i * period + dxMax >= x);
        }

        public static double Sec(double x)
        {
            bool negative;

            negative = IsInKBoundaries(Math.PI / 2.0, Math.PI * 3.0 / 2.0, 2.0 * Math.PI, x);

            if (!LimitToPeriod(-Math.PI / 2.0, Math.PI / 2.0, ref x))
            {
                return double.NaN;
            }

            double Value = MetaCot(x, 1.0, x * x, 1.0 * 2.0, 1, 0.0, -1.0, InnerSec, IsEuler2NOutOfRange, Euler2N);

            if (negative)
            {
                return -Value;
            }
            else
            {
                return Value;
            }
        }

        public static double Csc(double x)
        {
            bool negative;

            negative = IsInKBoundaries(Math.PI, 2.0 * Math.PI, 2.0 * Math.PI, x);

            if (!LimitToPeriod(0.0, Math.PI, ref x))
            {
                return double.NaN;
            }

            double Value = MetaCot(x, 1.0 / x, x, 1.0 * 2.0, 1, 2.0, 1.0, InnerCot, IsBernoulli2NOutOfRange, Bernoulli2N);

            if (Value > 5.4)
            {
                return double.NaN;
            }

            if (negative)
            {
                return -Value;
            }
            else
            {
                return Value;
            }
        }

        private delegate void InnerMetaCotFunction(ref double currentValue, double powerOfX, double factorial, double twoToPower, double minusOne, int n, GetNumber getNumber);

        private static void InnerCot(ref double currentValue, double powerOfX, double factorial, double twoToPower, double minusOne, int n, GetNumber getNumber)
        {
            currentValue += (minusOne) * (twoToPower) * getNumber(n) * powerOfX / factorial;
        }

        private static void InnerSec(ref double currentValue, double powerOfX, double factorial, double twoToPower, double minusOne, int n, GetNumber getNumber)
        {
            currentValue += minusOne * getNumber(n) * powerOfX / factorial;
        }

        private static double MetaCot(double x, double previousValue, double powerOfx, double factorial, int n, double twoToPower, double minusOne, InnerMetaCotFunction innerFun, IsOutOfRange isOutOfRange, GetNumber getNumber)
        {
            double currentValue = previousValue;

            innerFun(ref currentValue, powerOfx, factorial, twoToPower, minusOne, n, getNumber);

            int i = n + 1;
            int j = 0;

            while (IsPrecisionNotReached(currentValue, previousValue))
            {
                if (isOutOfRange(i))
                {
                    break;
                }

                j = 2 * i;
                previousValue = currentValue;
                powerOfx *= x * x;
                factorial *= (j - 1) * j;
                twoToPower *= 2 * 2;
                minusOne *= -1.0;

                innerFun(ref currentValue, powerOfx, factorial, twoToPower, minusOne, i, getNumber);

                i++;
            }

            return currentValue;
        }

        public static double Cot(double x)
        {
            if (!LimitToPeriod(0.0, Math.PI, ref x))
            {
                return double.NaN;
            }

            double val = MetaCot(x, 1.0 / x, x, 1.0 * 2.0, 1, 2.0 * 2.0, -1.0, InnerCot, IsBernoulli2NOutOfRange, Bernoulli2N);
            if (val > 11.4)
            {
                return double.NaN;
            }
            return val;
        }

        private static double MetaSinus(double x, double previousValue, double powerOfx, double factorial, double singleFactorialN, int jAdd, int n, double twoToPower, InnerMetaSinusFunction innerFun)
        {
            double currentValue;

            currentValue = previousValue;
            innerFun(ref currentValue, powerOfx, factorial, singleFactorialN, twoToPower, n);

            int i = n + 1;
            int j = 0;

            while (IsPrecisionNotReached(currentValue, previousValue))
            {
                j = jAdd + 2 * i;

                previousValue = currentValue;
                powerOfx *= x * x;
                factorial *= (j - 1) * j;
                singleFactorialN *= i;
                twoToPower *= 2.0 * 2.0;

                innerFun(ref currentValue, powerOfx, factorial, singleFactorialN, twoToPower, i);

                i++;
            }

            return currentValue;
        }

        private delegate void InnerMetaSinusFunction(ref double currentValue, double powerOfX, double factorial, double singleFactorialN, double twoToPower, int n);

        private static void InnerSinh(ref double currentValue, double powerOfx, double factorial, double singleFactorialN, double twoToPower, int n)
        {
            currentValue += powerOfx / factorial;
        }

        public static double Sinh(double x)
        {
            return MetaSinus(x, x, x * x * x, 1.0 * 2.0 * 3.0, 0.0, 1, 1, 0, InnerSinh);
        }

        private static void InnerArcSin(ref double currentValue, double powerOfx, double factorial, double factorialSingleN, double twoToPower, int n)
        {
            currentValue += factorial * powerOfx / (factorialSingleN * factorialSingleN * (2 * n + 1.0) * twoToPower);
        }

        public static double ArcSin(double x)
        {
            if (x < -1.0 || x > 1.0)
            {
                return double.NaN;
            }

            return MetaSinus(x, x, x * x * x, 1.0 * 2.0, 1.0, 0, 1, 2.0 * 2.0, InnerArcSin);
        }

        public static double ArcCos(double x)
        {
            return Math.PI / 2.0 - ArcSin(x);
        }

        public static double Cosh(double x)
        {
            return MetaSinus(x, 1.0, x * x, 1.0 * 2.0, 0.0, 0, 1, 0, InnerSinh);
        }

        public static double Ln(double x)
        {
            if (x <= 0.0001)
            {
                return double.NaN;
            }

            double previousValue;
            double currentValue;
            double powerOfx;
            double fraction;

            powerOfx = (x - 1.0) / (x + 1.0);
            previousValue = 1.0;
            currentValue = previousValue + powerOfx;

            int i = 2;

            while (IsPrecisionNotReached(currentValue, previousValue))
            {
                fraction = 1.0 / (2 * i + 1.0);
                previousValue = currentValue;
                powerOfx *= ((x - 1.0) / (x + 1.0) * (x - 1.0) / (x + 1.0));

                currentValue += fraction * powerOfx;

                i++;
            }

            return 2.0 * currentValue;
        }

        public static double Exp(double x)
        {
            double previousValue;
            double currentValue;
            double powerOfx;
            double factorial;

            powerOfx = x;
            factorial = 1.0;
            previousValue = 1;
            currentValue = previousValue + powerOfx / factorial;

            int i = 2;

            while (IsPrecisionNotReached(currentValue, previousValue))
            {
                previousValue = currentValue;
                powerOfx *= x;
                factorial *= i;

                currentValue += powerOfx / factorial;

                i++;
            }

            return currentValue;
        }
    }
}
