﻿/*
The MIT License (MIT)
Copyright (c) 2015 Wojciech Fialkiewicz
 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Functions
{
    public class FunctionsAdapter
    {
        public enum FUN
        {
            SEC,
            CSC,
            TAN,
            FLOOR,
            CEIL,
            COT,
            ARCSIN,
            ARCCOS,
            SIN,
            COS,
            LN,
            EXP,
            SINH,
            COSH
        }

        private static double Fun(double x, FUN f)
        {
            switch (f)
            {
                case FUN.CSC:
                    return Functions.Csc(x);

                case FUN.SEC:
                    return Functions.Sec(x);

                case FUN.TAN:
                    return Functions.Tan(x);

                case FUN.FLOOR:
                    return Functions.Floor(x);

                case FUN.CEIL:
                    return Functions.Ceil(x);

                case FUN.COT:
                    return Functions.Cot(x);

                case FUN.ARCSIN:
                    return Functions.ArcSin(x);

                case FUN.ARCCOS:
                    return Functions.ArcCos(x);

                default:
                    return double.NaN;
            }
        }

        private static double Fun_FC(FUN f, double x, double? PreviousValue, out bool connect)
        {
            if (PreviousValue == null)
            {
                connect = true;

                return Fun(x, f);
            }
            else
            {
                double val = Fun(x, f);

                PreviousValue = Fun(PreviousValue.Value, f);

                if (val != PreviousValue)
                {
                    connect = false;
                }
                else
                {
                    connect = true;
                }

                if (double.IsNaN(val))
                    connect = false;

                return val;
            }
        }

        private static double Fun_halfPI(FUN f, double x, double? PreviousValue, out bool connect)
        {
            if (PreviousValue == null)
            {
                connect = true;

                return Fun(x, f);
            }
            else
            {
                double val = Fun(x, f);

                PreviousValue = Fun(PreviousValue.Value, f);

                if (Functions.Abs(val - PreviousValue.Value) > Math.PI / 2.0)
                {
                    connect = false;
                }
                else
                {
                    connect = true;
                }

                if (double.IsNaN(val))
                    connect = false;

                return val;
            }
        }

        private static double Fun(FUN f, double x, double? PreviousValue, out bool connect)
        {
            if (PreviousValue == null)
            {
                connect = true;

                return Fun(x, f);
            }
            else
            {
                double val = Fun(x, f);

                PreviousValue = Fun(PreviousValue.Value, f);

                if (Functions.Abs(val - PreviousValue.Value) > Math.PI)
                {
                    connect = false;
                }
                else
                {
                    connect = true;
                }

                if (double.IsNaN(val))
                    connect = false;

                return val;
            }
        }

        private static int PeriodNr(double dMin, double dMax, double x)
        {
            if (x > dMin && x < dMax)
            {
                return 0;
            }

            double dPeriodLength = dMax - dMin;
            int periodNr = 0;

            if (x < dMin)
            {
                while (x <= dMin)
                {
                    dMin = dMin - dPeriodLength;
                    periodNr--;
                }
            }
            else//x>dMax
            {
                while (x >= dMax)
                {
                    dMax = dMax + dPeriodLength;
                    periodNr++;
                }
            }
            return periodNr;
        }

        private static double Csc(double x, double? PreviousValue, out bool connect)
        {
            double val = Fun(FUN.CSC, x, PreviousValue, out connect);

            int xPeriod = PeriodNr(0, Math.PI, x);
            int xPrevPeriod = PeriodNr(0, Math.PI, PreviousValue.Value);

            if (xPeriod == xPrevPeriod)
            {
                connect = true;
            }
            else
            {
                connect = false;
            }
            return val;
        }

        private static double Sec(double x, double? PreviousValue, out bool connect)
        {
            double val = Fun(FUN.SEC, x, PreviousValue, out connect);

            int xPeriod = PeriodNr(Math.PI / 2.0, 3.0 * Math.PI / 2.0, x);
            int xPrevPeriod = PeriodNr(Math.PI / 2.0, 3.0 * Math.PI / 2.0, PreviousValue.Value);

            if (xPeriod == xPrevPeriod)
            {
                connect = true;
            }
            else
            {
                connect = false;
            }
            return val;
        }

        private static double Tan(double x, double? PreviousValue, out bool connect)
        {
            double val = Fun(FUN.TAN, x, PreviousValue, out connect);

            int xPeriod = PeriodNr(Math.PI / 2.0, 3.0 * Math.PI / 2.0, x);
            int xPrevPeriod = PeriodNr(Math.PI / 2.0, 3.0 * Math.PI / 2.0, PreviousValue.Value);

            if (xPeriod == xPrevPeriod)
            {
                connect = true;
            }
            else
            {
                connect = false;
            }
            return val;
        }

        private static double Sin(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Sin(x);
        }

        private static double Cos(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Cos(x);
        }

        private static double Sinh(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Sinh(x);
        }

        private static double Cosh(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Cosh(x);
        }

        private static double Ln(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Ln(x);
        }

        private static double Exp(double x, double? PreviousValue, out bool connect)
        {
            connect = true;
            return Functions.Exp(x);
        }

        private static double Floor(double x, double? PreviousValue, out bool connect)
        {
            return Fun_FC(FUN.FLOOR, x, PreviousValue, out connect);
        }

        private static double Ceil(double x, double? PreviousValue, out bool connect)
        {
            return Fun_FC(FUN.CEIL, x, PreviousValue, out connect);
        }

        private static double Cot(double x, double PreviousValue, out bool connect)
        {
            double val = Fun(FUN.COT, x, PreviousValue, out connect);

            int xPeriod = PeriodNr(0, Math.PI, x);
            int xPrevPeriod = PeriodNr(0, Math.PI, PreviousValue);

            if (xPeriod == xPrevPeriod)
            {
                connect = true;
            }
            else
            {
                connect = false;
            }
            return val;
        }

        private static double ArcSin(double x, double? PreviousValue, out bool connect)
        {
            return Fun_halfPI(FUN.ARCSIN, x, PreviousValue, out connect);
        }

        private static double ArcCos(double x, double? PreviousValue, out bool connect)
        {
            return Fun_halfPI(FUN.ARCCOS, x, PreviousValue, out connect);
        }

        private delegate double Function(double x, double? PreviousValue, out bool connect);

        public bool Connect;

        private List<double> m_PreviousValues;
        private List<double> m_PreviousUsedValues;
        private List<FUN> m_FunctionsCalculated;

        public void Init(List<double> PreviousValues)
        {
            m_PreviousValues = new List<double>();
            m_FunctionsCalculated = new List<FUN>();
            m_PreviousUsedValues = PreviousValues;
            Connect = true;
        }

        public List<double> GetPreviousValues()
        {
            return m_PreviousValues;
        }

        public List<FUN> GetCalculatedFunctionsOrder()
        {
            return m_FunctionsCalculated;
        }

        private static bool ConnectPoints(double prevX, double x, FUN fun)
        {
            bool connect;

            switch (fun)
            {
                case FUN.ARCCOS:
                    ArcCos(x, prevX, out connect);
                    break;

                case FUN.ARCSIN:
                    ArcSin(x, prevX, out connect);
                    break;

                case FUN.CEIL:
                    Ceil(x, prevX, out connect);
                    break;

                case FUN.COS:
                    Cos(x, prevX, out connect);
                    break;

                case FUN.COSH:
                    Cosh(x, prevX, out connect);
                    break;

                case FUN.COT:
                    Cot(x, prevX, out connect);
                    break;

                case FUN.CSC:
                    Csc(x, prevX, out connect);
                    break;

                case FUN.EXP:
                    Exp(x, prevX, out connect);
                    break;

                case FUN.FLOOR:
                    Floor(x, prevX, out connect);
                    break;

                case FUN.LN:
                    Ln(x, prevX, out connect);
                    break;

                case FUN.SEC:
                    Sec(x, prevX, out connect);
                    break;

                case FUN.SIN:
                    Sin(x, prevX, out connect);
                    break;

                case FUN.SINH:
                    Sinh(x, prevX, out connect);
                    break;

                case FUN.TAN:
                    Tan(x, prevX, out connect);
                    break;

                default:
                    connect = false;
                    break;
            }

            return connect;
        }

        public static bool ConnectTwoPoints(List<double> pointA, List<double> pointB, List<FUN> funs)
        {
            if (pointA == null || pointB == null)
            {
                return false;
            }
            for (int i = 0; i < pointA.Count; i++)
            {
                if (!ConnectPoints(pointA[i], pointB[i], funs[i]))
                {
                    return false;
                }
            }
            return true;
        }

        private double PreviousValue(double x)
        {
            if (m_PreviousUsedValues == null)
            {
                return x;
            }
            else
            {
                int nr = m_PreviousValues.Count();

                return m_PreviousUsedValues[nr];
            }
        }

        private void AddPreviousValue(double val)
        {
            m_PreviousValues.Add(val);
        }

        private void AddCalculatedFunction(FUN f)
        {
            m_FunctionsCalculated.Add(f);
        }

        public double Csc(double x)
        {
            bool connect;

            double ret = Csc(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.CSC);
            return ret;
        }

        public double Sec(double x)
        {
            bool connect;

            double ret = Sec(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.SEC);
            return ret;
        }

        public double Tan(double x)
        {
            bool connect;

            double ret = Tan(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.TAN);
            return ret;
        }

        public double Sin(double x)
        {
            bool connect;

            double ret = Sin(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.SIN);
            return ret;
        }

        public double Cos(double x)
        {
            bool connect;

            double ret = Cos(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.COS);
            return ret;
        }

        public double Sinh(double x)
        {
            bool connect;

            double ret = Sinh(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.SINH);
            return ret;
        }

        public double Cosh(double x)
        {
            bool connect;

            double ret = Cosh(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.COSH);
            return ret;
        }

        public double Ln(double x)
        {
            bool connect;

            double ret = Ln(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.LN);
            return ret;
        }

        public double Exp(double x)
        {
            bool connect;

            double ret = Exp(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.EXP);
            return ret;
        }

        public double Floor(double x)
        {
            bool connect;

            double ret = Floor(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.FLOOR);
            return ret;
        }

        public double Ceil(double x)
        {
            bool connect;

            double ret = Ceil(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.CEIL);
            return ret;
        }

        public double Cot(double x)
        {
            bool connect;

            double ret = Cot(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.COT);
            return ret;
        }

        public double ArcSin(double x)
        {
            bool connect;

            double ret = ArcSin(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.ARCSIN);
            return ret;
        }

        public double ArcCos(double x)
        {
            bool connect;

            double ret = ArcCos(x, PreviousValue(x), out connect);

            Connect &= connect;
            AddPreviousValue(x);
            AddCalculatedFunction(FUN.ARCCOS);
            return ret;
        }
    }
}
