﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Drawing;
using System.Windows.Forms;

namespace FunctionsVisualizer
{
    public partial class MainWindow : Form
    {
        private void MenuTextHide()
        {
            showParametersAreaToolStripMenuItem.Text = "Hide Parameters Area";     
        }

        private void MenuTextShow()
        {
            showParametersAreaToolStripMenuItem.Text = "Show Parameters Area";
        }

        private void MenuTextDisable()
        {
            showParametersAreaToolStripMenuItem.Enabled = false;
        }

        private void MenuTextEnable()
        {
            showParametersAreaToolStripMenuItem.Enabled = true;
        }

        public bool Created;

        public MainWindow()
        {
            this.IsMdiContainer = true;
            InitializeComponent();

            Created = true;

            string text;

            Check.RegistryCheck.ApplicationName = "FunctionsVisualizer";

            if(true)//Check.RegistryCheck.FullVersion)
            {
                return;
            }
            else if(Check.RegistryCheck.EvaluationVersion(out text))
            {
                MessageBox.Show(text);
                return;
            }
            else
            {
                MessageBox.Show(text);
                Created = false;
                base.Close();
                return;
            }
        }

        private void form_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (this.MdiChildren == null || this.MdiChildren.Length == 0 || this.MdiChildren.Length == 1)
            {
                MenuTextShow();
                MenuTextDisable();
            }
        }

        private void showParametersAreaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((this.ActiveMdiChild as IParameters).ShownParameters())
            {
                (this.ActiveMdiChild as IParameters).ShowParameters(false);
                MenuTextShow();
            }
            else
            {
                (this.ActiveMdiChild as IParameters).ShowParameters(true);
                MenuTextHide();
            }
        }

        private void SwitchMenuText(object ob)
        {
            if ((ob as IParameters).ShownParameters())
            {
                MenuTextHide();
            }
            else
            {
                MenuTextShow();
            }
        }

        private void MainWindow_MdiChildActivate(object sender, EventArgs e)
        {
            if (this.ActiveMdiChild != null)
            {
                SwitchMenuText(this.ActiveMdiChild);
            }
        }

        private void dTimeFunctionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OneDimensionWithTime form = new OneDimensionWithTime();

            form.MdiParent = this;
            form.FormClosed += form_FormClosed;
            MenuTextEnable();
            form.Show();

            SwitchMenuText(form);
        }

        public static Pen FUNCTION_PEN;
        public static Pen INFO_PEN;
        public static Brush INFO_BRUSH;

        static MainWindow()
        {
            FUNCTION_PEN = Pens.DarkGoldenrod;
            INFO_BRUSH = Brushes.DarkGreen;
        }

        private void dTimeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TwoDimensionsWithTime form = new TwoDimensionsWithTime();

            form.MdiParent = this;
            form.FormClosed += form_FormClosed;
            MenuTextEnable();
            form.Show();

            SwitchMenuText(form);
        }

        private void dTimeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ThreeDimensionsWithTime form = new ThreeDimensionsWithTime();

            form.MdiParent = this;
            form.FormClosed += form_FormClosed;
            MenuTextEnable();
            form.Show();

            SwitchMenuText(form);
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            AboutForm about = new AboutForm();

            about.ShowDialog();
        }

        private void mathToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(Consts.FUNCTIONS, "Avaliable functions");
        }
    }
}
