﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using Functions;
using PointsManager;
using PointsManager._1D;
using PointsManager._2D;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;

namespace FunctionsVisualizer
{
    public partial class OneDimensionWithTime : Form, IParameters
    {
        private const int FPS = 24;
        private Timer timer;

        private double m_dTimeValue;

        private double m_dXBeg, m_dXEnd, m_dYBeg, m_dYEnd;
        private int m_iPointsAmount;
        private bool m_bShownParameters;
        private double m_dTBeg, m_dTEnd;
        private int m_iTimePointsAmount;
        private Line2D m_Lines;
        private MathFun m_Fun = new MathFun();
        private Pen m_FunPen;
        NumberFormatInfo m_format = new NumberFormatInfo();

        public OneDimensionWithTime()
        {
            InitializeComponent();
            m_Fun.SetFunctionBody1DT(FunctionsCompiler.Compiler.GetCompiledFunction1DT(textBox5.Text));
            
            m_format.NumberDecimalSeparator = ".";

            m_dXBeg = Double.Parse(xBeg.Text, m_format) * Math.PI;
            m_dXEnd = Double.Parse(xEnd.Text, m_format) * Math.PI;
            m_dYBeg = Double.Parse(yBeg.Text, m_format) * Math.PI;
            m_dYEnd = Double.Parse(yEnd.Text, m_format) * Math.PI;
            m_dTBeg = Double.Parse(textBox1.Text, m_format) * Math.PI;
            m_dTEnd = Double.Parse(textBox2.Text, m_format) * Math.PI;
            m_iPointsAmount = (int)(Double.Parse(textBox3.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iTimePointsAmount = (int)(Double.Parse(textBox4.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_bShownParameters = true;
            m_dTimeValue = m_dTBeg;
            m_FunPen = MainWindow.FUNCTION_PEN;
            ShowParameters(false);

            this.DoubleBuffered = true;

            timer = new Timer();
            timer.Tick += timer_Tick;
            timer.Interval = 1000 / FPS;
            timer.Start();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            double timeStep = (m_dTEnd - m_dTBeg) / (m_iTimePointsAmount - 1);

            m_dTimeValue += timeStep;

            if (m_dTimeValue >= m_dTEnd)
            {
                m_dTimeValue = m_dTBeg;
            }
            RefreshPictureBox();
        }

        private bool DrawAxes(Graphics g)
        {
            int width, height;

            width = pictureBox1.Width;
            height = pictureBox1.Height;

            g.FillRectangle(Brushes.Black, 0, 0, width, height);
            if (chShowTime.Checked)
            {
                g.DrawString("time : " + m_dTimeValue.ToString("F3"), SystemFonts.CaptionFont, MainWindow.INFO_BRUSH, new Point(0, 0));
            }
            return true;
        }

        private bool CalculateFunctionPoints(double time)
        {
            if (m_Fun.CanCalculate)
            {
                OneDimFunctionArguments args = new OneDimFunctionArguments();

                args.dXBeg = m_dXBeg;
                args.dXEnd = m_dXEnd;
                args.dYBeg = m_dYBeg;
                args.dYEnd = m_dYEnd;
                args.Height = pictureBox1.Height;
                args.Width = pictureBox1.Width;
                args.PointsAmountX = m_iPointsAmount;
                args.Time = time;
                args.Type = FunctionType.DTIME;
                args.Fun = m_Fun;

                m_Lines = new Line2D();
                m_Lines.CalculatePoints(args);
                return true;
            }
            return false;
        }

        private bool DrawFunction(Graphics g)
        {
            List<List<RefPoint>> list;

            CalculateFunctionPoints(m_dTimeValue);
            list = m_Lines.GetLines();
            for (int i = 0; i < list.Count; i++)
            {
                try
                {
                    g.DrawLine(m_FunPen, list[i][0].P, list[i][1].P);
                }
                catch(Exception ex)
                {

                }
            }
            return true;
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            base.OnPaint(e);

            DrawAxes(e.Graphics);
            DrawFunction(e.Graphics);
        }

        private void xBeg_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xBeg.Text, NumberStyles.Float, m_format, out m_dXBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXBeg *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void xEnd_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xEnd.Text, NumberStyles.Float, m_format, out m_dXEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXEnd *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void yBeg_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(yBeg.Text, NumberStyles.Float, m_format, out m_dYBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYBeg *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void yEnd_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(yEnd.Text, NumberStyles.Float, m_format, out m_dYEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYEnd *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void textBox3_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(textBox3.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmount = p;
                    RefreshPictureBox();
                }
            }
        }

        private void RefreshPictureBox()
        {
            pictureBox1.Refresh();
        }

        private void OneDimension_SizeChanged(object sender, EventArgs e)
        {
            pictureBox1.Height = splitContainer1.Panel2.Height;
            pictureBox1.Width = splitContainer1.Panel2.Width;
            RefreshPictureBox();
        }

        private void chShowAxes_CheckedChanged(object sender, EventArgs e)
        {
            RefreshPictureBox();
        }

        public bool ShownParameters()
        {
            return m_bShownParameters;
        }

        public void ShowParameters(bool show)
        {
            if (show)
            {
                splitContainer1.Panel1Collapsed = false;
                m_bShownParameters = true;
            }
            else
            {
                splitContainer1.Panel1Collapsed = true;
                m_bShownParameters = false;
            }
        }

        private void textBox4_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(textBox4.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iTimePointsAmount = p;
                    RefreshPictureBox();
                }
            }
        }

        private void textBox2_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textBox2.Text, NumberStyles.Float, m_format, out m_dTEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTEnd *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void textBox1_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textBox1.Text, NumberStyles.Float, m_format, out m_dTBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTBeg *= Math.PI;
                RefreshPictureBox();
            }
        }

        private void OneDimensionWithTime_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer.Stop();
            timer.Dispose();
        }

        private void textBox5_Validating(object sender, CancelEventArgs e)
        {
            Function1DT fun = FunctionsCompiler.Compiler.GetCompiledFunction1DT(textBox5.Text);

            if (fun == null)
            {
                e.Cancel = true;
            }
            else
            {
                m_Fun.SetFunctionBody1DT(fun);
                RefreshPictureBox();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ColorDialog dialog = new ColorDialog();

            dialog.ShowDialog();
            m_FunPen = new Pen(dialog.Color);
            RefreshPictureBox();
        }
    }
}
