﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using Functions;
using PointsManager;
using PointsManager._2D;
using PointsManager._3D;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;

namespace FunctionsVisualizer
{
    public partial class ThreeDimensionsWithTime : Form, IParameters
    {
        private const int FPS = 24;
        private Timer timer;

        private double m_dAngleXValue;
        private double m_dAngleYValue;
        private double m_dAngleZValue;

        private double YMovePercentMove;

        private double m_dXBeg, m_dXEnd, m_dZBeg, m_dZEnd, m_dYBeg, m_dYEnd, m_dQBeg, m_dQEnd;
        private int m_iPointsAmountX;
        private int m_iPointsAmountZ;
        private int m_iPointsAmountY;
        private bool m_bShownParameters;
        private int m_iAngleYPointsAm;
        private int m_iAngleZPointsAm;
        private int m_iAngleXPointsAm;
        private double m_dAngleXBeg, m_dAngleXEnd;
        private double m_dAngleYBeg, m_dAngleYEnd;
        private double m_dAngleZBeg, m_dAngleZEnd;
        private Mesh3D m_Scene;
        private double m_dTBeg, m_dTEnd;
        private int m_iTimePointsAm;
        private double m_dTimeValue;
        private MathFun m_Fun = new MathFun();
        private Pen m_FunPen;
        NumberFormatInfo m_format = new NumberFormatInfo();

        public ThreeDimensionsWithTime()
        {
            InitializeComponent();
            m_format.NumberDecimalSeparator = ".";
            m_Fun.SetFunctionBody3DT(FunctionsCompiler.Compiler.GetCompiledFunction3DT(textBox5.Text));
            m_dAngleXValue = 0.0;
            m_dAngleYValue = 0.0;
            m_dAngleZValue = 0.0;
            m_dTimeValue = 0.0;
            m_bShownParameters = true;
            m_dXBeg = Double.Parse(xMin.Text, m_format) * Math.PI;
            m_dXEnd = Double.Parse(xMax.Text, m_format) * Math.PI;
            m_dZBeg = Double.Parse(zMin.Text, m_format) * Math.PI;
            m_dZEnd = Double.Parse(zMax.Text, m_format) * Math.PI;
            m_dYBeg = Double.Parse(yMin.Text, m_format) * Math.PI;
            m_dYEnd = Double.Parse(yMax.Text, m_format) * Math.PI;
            m_dQBeg = Double.Parse(qMin.Text, m_format) * Math.PI;
            m_dQEnd = Double.Parse(qMax.Text, m_format) * Math.PI;
            m_iPointsAmountX = (int)(Double.Parse(xPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iPointsAmountZ = (int)(Double.Parse(zPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iPointsAmountY = (int)(Double.Parse(yPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iAngleXPointsAm = (int)(Double.Parse(xPointsLength.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iAngleYPointsAm = (int)(Double.Parse(yPointsLength.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iAngleZPointsAm = (int)(Double.Parse(zPointsLength.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_dAngleXBeg = 0.0;
            m_dAngleXEnd = 2.0 * Math.PI;
            m_dAngleYBeg = 0.0;
            m_dAngleYEnd = 2.0 * Math.PI;
            m_dAngleZBeg = 0.0;
            m_dAngleZEnd = 2.0 * Math.PI;
            m_dTBeg = Double.Parse(tMin.Text, m_format) * Math.PI;
            m_dTEnd = Double.Parse(tMax.Text, m_format) * Math.PI;
            m_iTimePointsAm = (int)(Double.Parse(tPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_FunPen = MainWindow.FUNCTION_PEN;
            ShowParameters(false);

            YMovePercentMove = 1.3 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
            textYMove.Text = YMovePercentMove.ToString(m_format);

            base.DoubleBuffered = true;

            CreateScene();

            timer = new Timer();
            timer.Tick += timer_Tick;
            timer.Interval = 1000 / FPS;
            timer.Start();
        }

        private void CreateScene()
        {
            if (m_Fun.CanCalculate)
            {
                ThreeDimFunctionArguments arg = new ThreeDimFunctionArguments();
                arg.dXBeg = m_dXBeg;
                arg.dXEnd = m_dXEnd;
                arg.dYBeg = m_dYBeg;
                arg.dYEnd = m_dYEnd;
                arg.dZBeg = m_dZBeg;
                arg.dZEnd = m_dZEnd;
                arg.QMin = m_dQBeg;
                arg.QMax = m_dQEnd;
                arg.Height = pictureBox1.Height;
                arg.Width = pictureBox1.Width;
                arg.Time = m_dTimeValue;
                arg.Type = FunctionType.DTIME;
                arg.PointsAmountX = m_iPointsAmountX;
                arg.PointsAmountY = m_iPointsAmountY;
                arg.PointsAmountZ = m_iPointsAmountZ;
                arg.Fun = m_Fun;

                m_Scene = new Mesh3D(m_iPointsAmountX, m_iPointsAmountZ, m_iPointsAmountY);
                m_Scene.CalculateMesh(arg);
            }
        }

        private void RefreshPictureBox()
        {
            CreateScene();
            pictureBox1.Refresh();
        }

        void timer_Tick(object sender, EventArgs e)
        {
            if (chRotateXAxis.Checked)
            {
                double timeStep = (m_dAngleXEnd - m_dAngleXBeg) / (m_iAngleXPointsAm - 1);

                m_dAngleXValue += timeStep;

                if (m_dAngleXValue >= m_dAngleXEnd)
                {
                    m_dAngleXValue = m_dAngleXBeg;
                }
            }

            if (chRotateYAxis.Checked)
            {
                double timeStep = (m_dAngleYEnd - m_dAngleYBeg) / (m_iAngleYPointsAm - 1);

                m_dAngleYValue += timeStep;

                if (m_dAngleYValue >= m_dAngleYEnd)
                {
                    m_dAngleYValue = m_dAngleYBeg;
                }
            }

            if (chRotateZAxis.Checked)
            {
                double timeStep = (m_dAngleZEnd - m_dAngleZBeg) / (m_iAngleZPointsAm - 1);

                m_dAngleZValue += timeStep;

                if (m_dAngleZValue >= m_dAngleZEnd)
                {
                    m_dAngleZValue = m_dAngleZBeg;
                }
            }

            double ts = (m_dTEnd - m_dTBeg) / (m_iTimePointsAm - 1);

            m_dTimeValue += ts;

            if (m_dTimeValue >= m_dTEnd)
            {
                m_dTimeValue = m_dTBeg;
            }

            RefreshPictureBox();
        }

        private void xMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xMin.Text, NumberStyles.Float, m_format, out m_dXBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void xMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xMax.Text, NumberStyles.Float, m_format, out m_dXEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void zMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(zMin.Text, NumberStyles.Float, m_format, out m_dZBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dZBeg *= (Math.PI);
                YMovePercentMove = 1.2 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
                textYMove.Text = YMovePercentMove.ToString();
                RefreshPictureBox();
            }
        }

        private void zMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(zMax.Text, NumberStyles.Float, m_format, out m_dZEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dZEnd *= (Math.PI);
                YMovePercentMove = 1.2 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
                textYMove.Text = YMovePercentMove.ToString(m_format);
                RefreshPictureBox();
            }
        }

        private void qMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(qMin.Text, NumberStyles.Float, m_format, out m_dQBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dQBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void qMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(qMax.Text, NumberStyles.Float, m_format, out m_dQEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dQEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void xPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(xPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2 || tmp > 1.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmountX = p;
                    RefreshPictureBox();
                }
            }
        }

        private void zPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(zPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2 || tmp > 1.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmountZ = p;
                    RefreshPictureBox();
                }
            }
        }

        public bool ShownParameters()
        {
            return m_bShownParameters;
        }

        public void ShowParameters(bool show)
        {
            if (show)
            {
                splitContainer1.Panel1Collapsed = false;
                m_bShownParameters = true;
            }
            else
            {
                splitContainer1.Panel1Collapsed = true;
                m_bShownParameters = false;
            }
            RefreshPictureBox();
        }

        private void tPointsLenght_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(xPointsLength.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iAngleXPointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void DrawSurface(Graphics g)
        {
            List<List<RefPoint>> lines;

            {
                m_Scene.Rotate(m_dAngleXValue, m_dAngleZValue, m_dAngleYValue);
                lines = m_Scene.GetLines(3.0, YMovePercentMove);
            }

            for (int i = 0; i < lines.Count; i++)
            {
                try
                {
                    g.DrawLine(m_FunPen, lines[i][0].P, lines[i][1].P);
                }
                catch(Exception ex)
                {

                }
            }
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            base.OnPaint(e);
            Bitmap bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics g = Graphics.FromImage(bmp);
            DrawSurface(g);
            e.Graphics.DrawImage(bmp, new Point(0, 0));
        }

        private void textDistance_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textYMove.Text, NumberStyles.Float, m_format, out YMovePercentMove))
            {
                e.Cancel = true;
                return;
            }
            RefreshPictureBox();
        }

        private void ThreeDimensionsWithTime_ClientSizeChanged(object sender, EventArgs e)
        {
            pictureBox1.Height = splitContainer1.Panel2.Height;
            pictureBox1.Width = splitContainer1.Panel2.Width;
            RefreshPictureBox();
        }

        private void yMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(yMin.Text, NumberStyles.Float, m_format, out m_dYBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void yMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(qMax.Text, NumberStyles.Float, m_format, out m_dYEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void zPointsLength_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(zPointsLength.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iAngleZPointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void yPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(yPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2 || tmp > 1.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmountY = p;
                    RefreshPictureBox();
                }
            }
        }

        private void yPointsLength_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(yPointsLength.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iAngleYPointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void tMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(tMin.Text, NumberStyles.Float, m_format, out m_dTBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void tMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(tMax.Text, NumberStyles.Float, m_format, out m_dTEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void tPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(tPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iTimePointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void textBox5_Validating(object sender, CancelEventArgs e)
        {
            Function3DT fun = FunctionsCompiler.Compiler.GetCompiledFunction3DT(textBox5.Text);

            if (fun == null)
            {
                e.Cancel = true;
            }
            else
            {
                m_Fun.SetFunctionBody3DT(fun);
                RefreshPictureBox();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ColorDialog dialog = new ColorDialog();

            dialog.ShowDialog();
            m_FunPen = new Pen(dialog.Color);
            RefreshPictureBox();
        }
    }
}
