﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using Functions;
using PointsManager;
using PointsManager._2D;
using PointsManager._3D;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;

namespace FunctionsVisualizer
{
    public partial class TwoDimensionsWithTime : Form, IParameters
    {
        private const int FPS = 24;
        private Timer timer;

        private double m_dAngleValue;
        private double m_dTimeValue;

        private double m_dXBeg, m_dXEnd, m_dZBeg, m_dZEnd, m_dYBeg, m_dYEnd, m_dTBeg, m_dTEnd;
        private int m_iPointsAmountX;
        private int m_iPointsAmountZ;
        private bool m_bShownParameters;
        private int m_iAnglePointsAm;
        private double m_dAngleBeg, m_dAngleEnd;
        private Scene3D m_Scene;
        private int m_iTimePointsAm;
        private MathFun m_Fun = new MathFun();
        private Pen m_FunPen;

        private double YMovePercentMove;
        NumberFormatInfo m_format = new NumberFormatInfo();

        public TwoDimensionsWithTime()
        {
            InitializeComponent();
            m_format.NumberDecimalSeparator = ".";
            m_Fun.SetFunctionBody2DT(FunctionsCompiler.Compiler.GetCompiledFunction2DT(textBox5.Text));
            m_dAngleValue = 0.0;
            m_bShownParameters = true;
            m_dXBeg = Double.Parse(xMin.Text, m_format) * Math.PI;
            m_dXEnd = Double.Parse(xMax.Text, m_format) * Math.PI;
            m_dZBeg = Double.Parse(zMin.Text, m_format) * Math.PI;
            m_dZEnd = Double.Parse(zMax.Text, m_format) * Math.PI;
            m_dYBeg = Double.Parse(yMin.Text, m_format) * Math.PI;
            m_dYEnd = Double.Parse(yMax.Text, m_format) * Math.PI;
            m_dTBeg = Double.Parse(textTMin.Text, m_format) * Math.PI;
            m_dTEnd = Double.Parse(textTMax.Text, m_format) * Math.PI;
            m_iPointsAmountX = (int)(Double.Parse(xPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iPointsAmountZ = (int)(Double.Parse(zPointsAm.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iAnglePointsAm = (int)(Double.Parse(tPointsLenght.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_iTimePointsAm = (int)(Double.Parse(textTimeLength.Text, m_format) * InnerDataPoint3D.ANGLE_OF_PI);
            m_dAngleBeg = 0.0;
            m_dAngleEnd = 2.0 * Math.PI;
            m_FunPen = MainWindow.FUNCTION_PEN;
            ShowParameters(false);

            YMovePercentMove = 1.2 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
            textYMove.Text = YMovePercentMove.ToString(m_format);

            base.DoubleBuffered = true;

            CreateScene();

            timer = new Timer();
            timer.Tick += timer_Tick;
            timer.Interval = 1000 / FPS;
            timer.Start();
        }

        private void CreateScene()
        {
            if (m_Fun.CanCalculate)
            {
                m_Scene = new Scene3D(m_iPointsAmountX, m_iPointsAmountZ);
                TwoDimFunctionArguments arg = new TwoDimFunctionArguments();
                arg.dXBeg = m_dXBeg;
                arg.dXEnd = m_dXEnd;
                arg.dYBeg = m_dYBeg;
                arg.dYEnd = m_dYEnd;
                arg.dZBeg = m_dZBeg;
                arg.dZEnd = m_dZEnd;
                arg.Height = pictureBox1.Height;
                arg.Width = pictureBox1.Width;
                arg.PointsAmountX = m_iPointsAmountX;
                arg.PointsAmountZ = m_iPointsAmountZ;
                arg.Type = FunctionType.DTIME;
                arg.Time = m_dTimeValue;
                arg.Fun = m_Fun;

                m_Scene.CalculateSurface(arg);
            }
        }

        private void RefreshPictureBox()
        {
            CreateScene();
            pictureBox1.Refresh();
        }

        void timer_Tick(object sender, EventArgs e)
        {
            if (chRotateYAxis.Checked)
            {
                double timeStep = (m_dAngleEnd - m_dAngleBeg) / (m_iAnglePointsAm - 1);

                m_dAngleValue += timeStep;

                if (m_dAngleValue >= m_dAngleEnd)
                {
                    m_dAngleValue = m_dAngleBeg;
                }
            }

            double ts = (m_dTEnd - m_dTBeg) / (m_iTimePointsAm - 1);

            m_dTimeValue += ts;

            if (m_dTimeValue >= m_dTEnd)
            {
                m_dTimeValue = m_dTBeg;
            }
            RefreshPictureBox();
        }

        private void xMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xMin.Text, NumberStyles.Float, m_format, out m_dXBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void xMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(xMax.Text, NumberStyles.Float, m_format, out m_dXEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dXEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void zMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(zMin.Text, NumberStyles.Float, m_format, out m_dZBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dZBeg *= (Math.PI);
                YMovePercentMove = 1.2 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
                textYMove.Text = YMovePercentMove.ToString();
                RefreshPictureBox();
            }
        }

        private void zMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(zMax.Text, NumberStyles.Float, m_format, out m_dZEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dZEnd *= (Math.PI);
                YMovePercentMove = 1.2 / ((m_dZEnd - m_dZBeg) / (Math.PI * 2));
                textYMove.Text = YMovePercentMove.ToString(m_format);
                RefreshPictureBox();
            }
        }

        private void yMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(yMin.Text, NumberStyles.Float, m_format, out m_dYBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void yMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(yMax.Text, NumberStyles.Float, m_format, out m_dYEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dYEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void xPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(xPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2 || tmp > 3.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmountX = p;
                    RefreshPictureBox();
                }
            }
        }

        private void zPointsAm_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(zPointsAm.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                int p = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);

                if (p < 2 || tmp > 3.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iPointsAmountZ = p;
                    RefreshPictureBox();
                }
            }
        }

        public bool ShownParameters()
        {
            return m_bShownParameters;
        }

        public void ShowParameters(bool show)
        {
            if (show)
            {
                splitContainer1.Panel1Collapsed = false;
                m_bShownParameters = true;
            }
            else
            {
                splitContainer1.Panel1Collapsed = true;
                m_bShownParameters = false;
            }
            RefreshPictureBox();
        }

        private void tPointsLenght_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(tPointsLenght.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0 || tmp > 20.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iAnglePointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void DrawTest(Graphics g)
        {
            Point3D a, b, c, d;

            InnerDataPoint3D inner = new InnerDataPoint3D();

            inner.Screen_Height = pictureBox1.Height;
            inner.Screen_Width = pictureBox1.Width;
            inner.X_Min = m_dXBeg;
            inner.X_Max = m_dXEnd;
            inner.Y_Min = m_dYBeg;
            inner.Y_Max = m_dYEnd;
            inner.Z_Min = m_dZBeg;
            inner.Z_Max = m_dZEnd;
            inner.CalculateScales();

            a = new Point3D(inner);
            b = new Point3D(inner);
            c = new Point3D(inner);
            d = new Point3D(inner);

            double dX, dZ;

            dX = 0.0;// (m_dXEnd - m_dXBeg) / 7;
            dZ = 0.0;// (m_dZEnd - m_dZBeg) / 7;

            a.X = m_dXBeg + (m_dXEnd - m_dXBeg) / 3 - dX;
            a.Y = m_dYBeg + (m_dYEnd - m_dYBeg) / 3;
            a.Z = m_dZBeg + (m_dZEnd - m_dZBeg) / 3 + dZ;

            b.X = m_dXBeg + 2 * (m_dXEnd - m_dXBeg) / 3 - dX;
            b.Y = m_dYBeg + (m_dYEnd - m_dYBeg) / 3;
            b.Z = m_dZBeg + (m_dZEnd - m_dZBeg) / 3 - dZ;

            c.X = m_dXBeg + 2 * (m_dXEnd - m_dXBeg) / 3 + dX;
            c.Y = m_dYBeg + (m_dYEnd - m_dYBeg) / 3;
            c.Z = m_dZBeg + 2 * (m_dZEnd - m_dZBeg) / 3 - dZ;

            d.X = m_dXBeg + (m_dXEnd - m_dXBeg) / 3 + dX;
            d.Y = m_dYBeg + (m_dYEnd - m_dYBeg) / 3;
            d.Z = m_dZBeg + 2 * (m_dZEnd - m_dZBeg) / 3 + dZ;

            a = Point3D.RotateY(a, m_dAngleValue);
            b = Point3D.RotateY(b, m_dAngleValue);
            c = Point3D.RotateY(c, m_dAngleValue);
            d = Point3D.RotateY(d, m_dAngleValue);

            PointF a2D = a.Point2D;
            PointF b2D = b.Point2D;
            PointF c2D = c.Point2D;
            PointF d2D = d.Point2D;

            List<RefPoint> points = new List<RefPoint>();

            points.Add(new RefPoint(a2D));
            points.Add(new RefPoint(b2D));
            points.Add(new RefPoint(c2D));
            points.Add(new RefPoint(d2D));

            a.FitPointsToScreen(points, 3.0, YMovePercentMove);

            g.DrawString("a", SystemFonts.CaptionFont, MainWindow.INFO_BRUSH, points[0].P);
            g.DrawString("b", SystemFonts.CaptionFont, MainWindow.INFO_BRUSH, points[1].P);
            g.DrawString("c", SystemFonts.CaptionFont, MainWindow.INFO_BRUSH, points[2].P);
            g.DrawString("d", SystemFonts.CaptionFont, MainWindow.INFO_BRUSH, points[3].P);

            g.DrawLine(MainWindow.FUNCTION_PEN, points[0].P, points[1].P);
            g.DrawLine(MainWindow.FUNCTION_PEN, points[1].P, points[2].P);
            g.DrawLine(MainWindow.FUNCTION_PEN, points[2].P, points[3].P);
            g.DrawLine(MainWindow.FUNCTION_PEN, points[3].P, points[0].P);
        }

        private void DrawSurface(Graphics g)
        {
            m_Scene.Rotate(m_dAngleValue);
            List<List<RefPoint>> lines = m_Scene.GetLines(3.0, YMovePercentMove);

            for (int i = 0; i < lines.Count; i++)
            {
                try
                {
                    if (!float.IsNaN(lines[i][0].P.X) && !float.IsNaN(lines[i][0].P.Y) && !float.IsNaN(lines[i][1].P.X) && !float.IsNaN(lines[i][1].P.Y))
                    {
                        g.DrawLine(m_FunPen, lines[i][0].P, lines[i][1].P);
                    }
                }
                catch (Exception ex)
                {

                }
            }
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            base.OnPaint(e);
            Bitmap bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics g = Graphics.FromImage(bmp);
            DrawSurface(g);
            e.Graphics.DrawImage(bmp, new Point(0, 0));
        }

        private void textDistance_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textYMove.Text, NumberStyles.Float, m_format, out YMovePercentMove))
            {
                e.Cancel = true;
                return;
            }
            RefreshPictureBox();
        }

        private void TwoDimensionsWithTime_ClientSizeChanged(object sender, EventArgs e)
        {
            pictureBox1.Height = splitContainer1.Panel2.Height;
            pictureBox1.Width = splitContainer1.Panel2.Width;
            RefreshPictureBox();
        }

        private void textTMin_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textTMin.Text, NumberStyles.Float, m_format, out m_dTBeg))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTBeg *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void textTMax_Validating(object sender, CancelEventArgs e)
        {
            if (!Double.TryParse(textTMax.Text, NumberStyles.Float, m_format, out m_dTEnd))
            {
                e.Cancel = true;
            }
            else
            {
                m_dTEnd *= (Math.PI);
                RefreshPictureBox();
            }
        }

        private void textTimeLength_Validating(object sender, CancelEventArgs e)
        {
            double tmp;

            if (!Double.TryParse(textTimeLength.Text, NumberStyles.Float, m_format, out tmp))
            {
                e.Cancel = true;
            }
            else
            {
                if (tmp < 0.0 || tmp > 20.0)
                {
                    e.Cancel = true;
                }
                else
                {
                    m_iTimePointsAm = (int)(tmp * InnerDataPoint3D.ANGLE_OF_PI);
                    RefreshPictureBox();
                }
            }
        }

        private void TwoDimensionsWithTime_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer.Stop();
            timer.Dispose();
        }

        private void textBox5_Validating(object sender, CancelEventArgs e)
        {
            Function2DT fun = FunctionsCompiler.Compiler.GetCompiledFunction2DT(textBox5.Text);

            if (fun == null)
            {
                e.Cancel = true;
            }
            else
            {
                m_Fun.SetFunctionBody2DT(fun);
                RefreshPictureBox();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ColorDialog dialog = new ColorDialog();

            dialog.ShowDialog();
            m_FunPen = new Pen(dialog.Color);
            RefreshPictureBox();
        }
    }
}
