﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System;
using System.Collections.Generic;
using System.Drawing;

namespace PointsManager._2D
{
    public class Point3D
    {
        protected InnerDataPoint3D m_Data;

        public double X { get; set; }
        public double Y { get; set; }
        public double Z { get; set; }

        protected double dYMove { get; set; }
        protected double dZMove { get; set; }

        public Point3D(Point3D p)
        {
            X = p.X;
            Y = p.Y;
            Z = p.Z;
            m_Data = p.m_Data;
            dYMove = p.dYMove;
            dZMove = p.dZMove;
        }

        public Point3D(InnerDataPoint3D data)
        {
            m_Data = data;
            dYMove = 1.7 * (m_Data.Y_Max - m_Data.Y_Min);
            dZMove = 3 * (m_Data.Z_Max - m_Data.Z_Min);
        }

        public static Point3D Rotate(Point3D point, double angleX, double angleZ, double angleY)
        {
            double s = Math.Sin(angleY);
            double c = Math.Cos(angleY);

            Point3D p = new Point3D(point);

            p.X = point.X * c + point.Z * s;
            p.Z = point.X * (-s) + point.Z * c;
            p.Y = point.Y;

            double sX = Math.Sin(angleX);
            double cX = Math.Cos(angleX);

            Point3D p2 = new Point3D(p);

            p2.Z = p.Z * cX + p.Y * sX;
            p2.Y = p.Z * (-sX) + p.Y * cX;
            p2.X = p.X;

            double sZ = Math.Sin(angleZ);
            double cZ = Math.Cos(angleZ);

            Point3D p3 = new Point3D(p2);

            p3.X = p2.X * cZ + p2.Y * sZ;
            p3.Y = p2.X * (-sZ) + p2.Y * cZ;
            p3.Z = p2.Z;
                        
            return p3;
        }

        public static Point3D RotateY(Point3D point, double angleY)
        {
            double s = Math.Sin(angleY);
            double c = Math.Cos(angleY);

            Point3D p = new Point3D(point);

            p.X = point.X * c + point.Z * s;
            p.Z = point.X * (-s) + point.Z * c;
            p.Y = point.Y;

            return p;
        }
               
        public PointF Point2D
        {
            get
            {
                PointF point;
                double x, y;

                x = X / (1.0 + Z + dZMove) * InnerDataPoint3D.ScreenDistance;
                y = (Y - dYMove) / (1.0 + Z + dZMove) * InnerDataPoint3D.ScreenDistance;
                point = new PointF((float)(m_Data.x_scaled_beg + m_Data.x_scale * x), (float)m_Data.Screen_Height - (float)(m_Data.y_scaled_beg + m_Data.y_scale * y));

                return point;
            }
        }

        public void FitPointsToScreen(List<RefPoint> points, double ScaleFactor, double YMove)
        {
            foreach (RefPoint p in points)
            {
                PointF pt = new PointF();

                pt.X = (float)(m_Data.Screen_Width / 2.0 - (m_Data.Screen_Width / 2.0 - p.P.X) * ScaleFactor);
                pt.Y = (float)(m_Data.Screen_Height / 2.0 - (m_Data.Screen_Height / 2.0 - p.P.Y) * ScaleFactor) - (int)(m_Data.Screen_Height * YMove);

                p.P = pt;
            }
        }
    }
}
