﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System.Collections.Generic;

namespace PointsManager._2D
{
    public class SurfacePoints3D
    {
        Point3D[,] m_Surface;
        List<double>[,] m_FunctionsArguments;

        public SurfacePoints3D(int am_x, int am_z)
        {
            m_Surface = new Point3D[am_x, am_z];
            m_FunctionsArguments = new List<double>[am_x, am_z];
            for (int x = 0; x < am_x; x++)
            {
                for (int z = 0; z < am_z; z++)
                {
                    m_Surface[x, z] = null;
                    m_FunctionsArguments[x, z] = null;
                }
            }
        }

        public int X_AM
        {
            get
            {
                return m_Surface.GetLength(0);
            }
        }

        public int Z_AM
        {
            get
            {
                return m_Surface.GetLength(1);
            }
        }

        public Point3D this[int x, int z]
        {
            get
            {
                return m_Surface[x, z];
            }
            set
            {
                m_Surface[x, z] = value;
            }
        }

        public List<double> GetArguments(int x, int z)
        {
            return m_FunctionsArguments[x, z];
        }
        
        public void SetArguments(int x, int z, List<double> arg)
        {
            m_FunctionsArguments[x, z] = arg;
        }

        public static SurfacePoints3D GetRotatedSurface(SurfacePoints3D surface, double angle)
        {
            SurfacePoints3D s = new SurfacePoints3D(surface.X_AM, surface.Z_AM);

            for (int x = 0; x < surface.X_AM; x++)
            {
                for (int z = 0; z < surface.Z_AM; z++)
                {
                    if (surface[x, z] != null)
                    {
                        s[x, z] = Point3D.RotateY(surface[x, z], angle);
                    }
                }
            }
            return s;
        }
    }
}
