﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using PointsManager._2D;
using System.Collections.Generic;

namespace PointsManager._3D
{
    public class Mesh3D
    {
        MeshPoints3D orginal_mesh;
        MeshPoints3D mesh;
        int m_x, m_z, m_y;
        List<Functions.FunctionsAdapter.FUN> m_funcs;

        public Mesh3D(int x, int z, int y)
        {
            m_x = x;
            m_z = z;
            m_y = y;
        }

        public void Rotate(double angleX, double angleZ, double angleY)
        {
            mesh = MeshPoints3D.GetRotatedMesh(orginal_mesh, angleX, angleZ, angleY);
        }

        public void CalculateMesh(ThreeDimFunctionArguments arg)
        {
            Tools.CalculateFunctionPoints(out orginal_mesh, arg, out m_funcs);
        }

        private void AddLine(int i, int j, int k, int i2, int j2, int k2)
        {
            if (tmp_mesh[i2][j2][k2] != null)
            {
                if (Functions.FunctionsAdapter.ConnectTwoPoints(orginal_mesh.GetFunctionArguments(i, j, k), orginal_mesh.GetFunctionArguments(i2, j2, k2), m_funcs))
                {
                    List<RefPoint> l = new List<RefPoint>();
                    l.Add(tmp_mesh[i][j][k]);
                    l.Add(tmp_mesh[i2][j2][k2]);
                    lines.Add(l);
                }
            }
        }

        List<List<List<RefPoint>>> tmp_mesh;
        List<RefPoint> list;
        List<List<RefPoint>> lines;

        public List<List<RefPoint>> GetLines(double ScaleFactor, double YMove)
        {
            list = new List<RefPoint>();
            RefPoint p;
            Point3D example = null;

            tmp_mesh = new List<List<List<RefPoint>>>();
            for (int i = 0; i < mesh.X_AM; i++)
            {
                tmp_mesh.Add(new List<List<RefPoint>>());
                for (int j = 0; j < mesh.Z_AM; j++)
                {
                    tmp_mesh[i].Add(new List<RefPoint>());
                    for (int k = 0; k < mesh.Y_AM; k++)
                    {
                        if (mesh[i, j, k] != null)
                        {
                            p = new RefPoint(mesh[i, j, k].Point2D);
                            tmp_mesh[i][j].Add(p);
                            list.Add(p);

                            if (example == null)
                            {
                                example = mesh[i, j, k];
                            }
                        }
                        else
                        {
                            tmp_mesh[i][j].Add(null);
                        }
                    }
                }
            }

            lines = new List<List<RefPoint>>();

            for (int i = 0; i < mesh.X_AM; i++)
            {
                for (int j = 0; j < mesh.Z_AM; j++)
                {
                    for (int k = 0; k < mesh.Y_AM; k++)
                    {
                        if (tmp_mesh[i][j][k] != null)
                        {
                            if (i < mesh.X_AM - 1 && j < mesh.Z_AM - 1 && k < mesh.Y_AM - 1)
                            {
                                AddLine(i, j, k, i, j, k + 1);
                                AddLine(i, j, k, i, j + 1, k);
                                AddLine(i, j, k, i + 1, j, k);
                            }
                            else
                            {
                                if (i < mesh.X_AM - 1 && j < mesh.Z_AM - 1 && k == mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i, j + 1, k);
                                    AddLine(i, j, k, i + 1, j, k);
                                }
                                else if (i == mesh.X_AM - 1 && j < mesh.Z_AM - 1 && k < mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i, j, k + 1);
                                    AddLine(i, j, k, i, j + 1, k);
                                }
                                else if (i < mesh.X_AM - 1 && j == mesh.Z_AM - 1 && k < mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i + 1, j, k);
                                    AddLine(i, j, k, i, j, k + 1);
                                }
                                else if (i == mesh.X_AM - 1 && j == mesh.Z_AM - 1 && k < mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i, j, k + 1);
                                }
                                else if (i < mesh.X_AM - 1 && j == mesh.Z_AM - 1 && k == mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i + 1, j, k);
                                }
                                else if (i == mesh.X_AM - 1 && j < mesh.Z_AM - 1 && k == mesh.Y_AM - 1)
                                {
                                    AddLine(i, j, k, i, j + 1, k);
                                }
                            }
                        }
                    }
                }
            }
            if (example != null)
            {
                example.FitPointsToScreen(list, ScaleFactor, YMove);
            }
            list = null;
            tmp_mesh = null;
            return lines;
        }
    }
}
