﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using PointsManager._2D;
using System.Collections.Generic;

namespace PointsManager._3D
{
    public class MeshPoints3D
    {
        Point3D[, ,] m_Mesh;
        List<double>[, ,] m_FunctionsArguments;
             
        public MeshPoints3D(int am_x, int am_z, int am_y)
        {
            m_Mesh = new Point3D[am_x, am_z, am_y];
        
            for (int x = 0; x < am_x; x++)
            {
                for (int z = 0; z < am_z; z++)
                {
                    for (int y = 0; y < am_y; y++)
                    {
                        m_Mesh[x, z, y] = null;
                    }
                }
            }
        }

        public int X_AM
        {
            get
            {
                return m_Mesh.GetLength(0);
            }
        }

        public int Z_AM
        {
            get
            {
                return m_Mesh.GetLength(1);
            }
        }

        public int Y_AM
        {
            get
            {
                return m_Mesh.GetLength(2);
            }
        }

        public List<double> GetFunctionArguments(int x, int z, int y)
        {
            return m_FunctionsArguments[x, z, y];
        }
                      
        public Point3D this[int x, int z, int y]
        {
            get
            {
                return m_Mesh[x, z, y];
            }
            set
            {
                m_Mesh[x, z, y] = value;
            }
        }

        public static MeshPoints3D GetMesh3D(MeshPoints4D mesh4D)
        {
            MeshPoints3D mesh3D = new MeshPoints3D(mesh4D.X_AM, mesh4D.Z_AM, mesh4D.Y_AM);

            mesh3D.m_FunctionsArguments = mesh4D.GetAllFunctionsArguments();

            for (int x = 0; x < mesh3D.X_AM; x++)
            {
                for (int z = 0; z < mesh3D.Z_AM; z++)
                {
                    for (int y = 0; y < mesh3D.Y_AM; y++)
                    {
                        if (mesh4D[x, z, y] != null && mesh4D[x, z, y].IsValid)
                        {
                            mesh3D[x, z, y] = mesh4D[x, z, y].Point3D;
                        }
                    }
                }
            }
            return mesh3D;
        }

        public static MeshPoints3D GetRotatedMesh(MeshPoints3D mesh, double angleX, double angleZ, double angleY)
        {
            MeshPoints3D m = new MeshPoints3D(mesh.X_AM, mesh.Z_AM, mesh.Y_AM);

            for (int x = 0; x < mesh.X_AM; x++)
            {
                for (int z = 0; z < mesh.Z_AM; z++)
                {
                    for (int y = 0; y < mesh.Y_AM; y++)
                    {
                        if (mesh[x, z, y] != null)
                        {
                            m[x, z, y] = Point3D.Rotate(mesh[x, z, y], angleX, angleZ, angleY);
                        }
                        else
                        {
                            m[x, z, y] = null;
                        }
                    }
                }
            }
            return m;
        }
    }
}
