﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using System.Collections.Generic;

namespace PointsManager._3D
{
    public class MeshPoints4D
    {
        Point4D[,,] m_Mesh;
        List<double>[, ,] m_FunctionsArguments;
       
        public MeshPoints4D(int am_x, int am_z, int am_y)
        {
            m_Mesh = new Point4D[am_x, am_z, am_y];
            m_FunctionsArguments = new List<double>[am_x, am_z, am_y];

            for (int x = 0; x < am_x; x++)
            {
                for (int z = 0; z < am_z; z++)
                {
                    for (int y = 0; y < am_y; y++)
                    {
                        m_Mesh[x, z, y] = null;
                    }
                }
            }
        }

        public void SetFunctionArguments(int x, int z, int y, List<double> args)
        {
            m_FunctionsArguments[x, z, y] = args;
        }
        
        public List<double>[, ,] GetAllFunctionsArguments()
        {
            return m_FunctionsArguments;
        }

        public int X_AM
        {
            get
            {
                return m_Mesh.GetLength(0);
            }
        }

        public int Z_AM
        {
            get
            {
                return m_Mesh.GetLength(1);
            }
        }

        public int Y_AM
        {
            get
            {
                return m_Mesh.GetLength(2);
            }
        }

        public Point4D this[int x, int z, int y]
        {
            get
            {
                return m_Mesh[x, z, y];
            }
            set
            {
                m_Mesh[x, z, y] = value;
            }
        }
    }
}
