﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using PointsManager._2D;

namespace PointsManager._3D
{
    public class Point4D : Point3D
    {
        private InnerDataPoint4D m_4DData;

        public Point4D(InnerDataPoint4D inner4D, InnerDataPoint3D inner3D)
            : base(inner3D)
        {
            m_4DData = inner4D;
            dQMove = (inner4D.Q_Max - m_4DData.Q_Min);
        }

        public Point4D(Point4D p)
            : base(p as Point3D)
        {
            m_4DData = p.m_4DData;
            Q = p.Q;
            dQMove = p.dQMove;
        }

        public double Q { get; set; }

        public double dQMove { get; set; }

        public bool IsValid
        {
            get
            {
                if (m_4DData.Q_Min < 0.0)
                {
                    return Q > 2.0 * m_4DData.Q_Min;
                }
                else
                {
                    return Q > 0.0;
                }
            }
        }

        public Point3D Point3D
        {
            get
            {
                Point3D point = new Point3D(base.m_Data);

                point.X = X / (1.0 + Q + dQMove) * InnerDataPoint3D.ScreenDistance;
                point.Y = Y / (1.0 + Q + dQMove) * InnerDataPoint3D.ScreenDistance;
                point.Z = Z / (1.0 + Q + dQMove) * InnerDataPoint3D.ScreenDistance;

                return point;
            }
        }
    }
}
