﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using Functions;
using PointsManager._2D;
using System.Collections.Generic;

namespace PointsManager._3D
{
    public class Scene3D
    {
        SurfacePoints3D orginal_surface;
        SurfacePoints3D surface;
        int m_x, m_z;
        List<Functions.FunctionsAdapter.FUN> m_funcs;

        public Scene3D(int x, int z)
        {
            m_x = x;
            m_z = z;
        }

        public void Rotate(double angle)
        {
            surface = SurfacePoints3D.GetRotatedSurface(orginal_surface, angle);
        }

        public void CalculateSurface(TwoDimFunctionArguments arg)
        {
            Tools.CalculateFunctionPoints(out orginal_surface, arg, out m_funcs);
        }

        public List<List<RefPoint>> GetLines(double ScaleFactor, double YMove)
        {
            List<List<RefPoint>> tmp_surf;
            List<RefPoint> list = new List<RefPoint>();
            RefPoint p;
            Point3D example = null;

            tmp_surf = new List<List<RefPoint>>();
            for (int i = 0; i < surface.X_AM; i++)
            {
                tmp_surf.Add(new List<RefPoint>());
                for (int j = 0; j < surface.Z_AM; j++)
                {
                    if (surface[i, j] != null)
                    {
                        p = new RefPoint(surface[i, j].Point2D);
                        tmp_surf[i].Add(p);
                        list.Add(p);
                        if (example == null)
                        {
                            example = surface[i, j];
                        }
                    }
                    else
                    {
                        tmp_surf[i].Add(null);
                    }
                }
            }

            List<List<RefPoint>> lines = new List<List<RefPoint>>();


            for (int i = 0; i < surface.X_AM; i++)
            {
                for (int j = 0; j < surface.Z_AM; j++)
                {
                    if (tmp_surf[i][j] != null)
                    {
                        if (i < surface.X_AM - 1 && j < surface.Z_AM - 1)
                        {
                            if (tmp_surf[i][j + 1] != null)
                            {
                                if (FunctionsAdapter.ConnectTwoPoints(orginal_surface.GetArguments(i, j), orginal_surface.GetArguments(i, j + 1), m_funcs))
                                {
                                    List<RefPoint> l = new List<RefPoint>();
                                    l.Add(tmp_surf[i][j]);
                                    l.Add(tmp_surf[i][j + 1]);
                                    lines.Add(l);
                                }
                            }
                            if (tmp_surf[i + 1][j] != null)
                            {
                                if (FunctionsAdapter.ConnectTwoPoints(orginal_surface.GetArguments(i, j), orginal_surface.GetArguments(i + 1, j), m_funcs))
                                {
                                    List<RefPoint> l = new List<RefPoint>();
                                    l.Add(tmp_surf[i][j]);
                                    l.Add(tmp_surf[i + 1][j]);
                                    lines.Add(l);
                                }
                            }
                        }
                        else
                        {
                            if (i < surface.X_AM - 1)
                            {
                                if (tmp_surf[i + 1][j] != null)
                                {
                                    if (FunctionsAdapter.ConnectTwoPoints(orginal_surface.GetArguments(i, j), orginal_surface.GetArguments(i + 1, j), m_funcs))
                                    {
                                        List<RefPoint> l = new List<RefPoint>();
                                        l.Add(tmp_surf[i][j]);
                                        l.Add(tmp_surf[i + 1][j]);
                                        lines.Add(l);
                                    }
                                }
                            }
                            if (j < surface.Z_AM - 1)
                            {
                                if (tmp_surf[i][j + 1] != null)
                                {
                                    if (FunctionsAdapter.ConnectTwoPoints(orginal_surface.GetArguments(i, j), orginal_surface.GetArguments(i, j + 1), m_funcs))
                                    {
                                        List<RefPoint> l = new List<RefPoint>();
                                        l.Add(tmp_surf[i][j]);
                                        l.Add(tmp_surf[i][j + 1]);
                                        lines.Add(l);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if (example != null)
            {
                example.FitPointsToScreen(list, 3.0, YMove);
            }
            return lines;
        }
    }
}
