﻿/*Copyright 2024 Wojciech Krzysztof Fiałkiewicz

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.*/

using PointsManager._1D;
using PointsManager._2D;
using PointsManager._3D;
using System.Collections.Generic;
using System.Drawing;

namespace PointsManager
{
    public class Tools
    {
        public static bool CalculateFunctionPoints(out List<List<RefPoint>> points, OneDimFunctionArguments arg)
        {
            points = new List<List<RefPoint>>();
            double x = arg.dXBeg;
            double y;
            double x_step;
            RefPoint point;
            RefPoint prevPoint = null;
            double x_scale, y_scale;
            double x_scaled_beg, y_scaled_beg;
            int height;
            bool is_value;
            bool connect = false;
            List<double> prevs = null;

            height = arg.Height;
            x_scale = arg.Width / (arg.dXEnd - arg.dXBeg);
            y_scale = height / (arg.dYEnd - arg.dYBeg);
            x_scaled_beg = (-arg.dXBeg) * x_scale;
            y_scaled_beg = (-arg.dYBeg) * y_scale;
            x_step = ((arg.dXEnd - arg.dXBeg) / (arg.PointsAmountX - 1));

            for (int i = 0; i < arg.PointsAmountX; i++)
            {
                if (arg.Type == FunctionType.D)
                {
                    is_value = false;
                    y = 0.0;
                }
                else
                {
                    if (arg.Fun.CalculateFunction1DT(ref prevs, x, arg.Time, out y, out connect))
                    {
                        is_value = true;
                    }
                    else
                    {
                        is_value = false;
                    }
                    if(!connect)
                    {
                        int j = 9;
                        j = 10;
                    }
                }
                if (is_value && connect)
                {
                    point = new RefPoint(new PointF((float)(x_scaled_beg + x_scale * x), (float)height - (float)(y_scaled_beg + y_scale * y)));
                    if(prevPoint != null)
                    {
                        List<RefPoint> line = new List<RefPoint>();

                        line.Add(prevPoint);
                        line.Add(point);

                        points.Add(line);
                    }
                    prevPoint = point;
                }
                else
                {
                    prevPoint = null;
//                    prevs = null;
                }
                x += x_step;
            }
            return true;
        }

        public static bool CalculateFunctionPoints(out SurfacePoints3D points, TwoDimFunctionArguments arg, out List<Functions.FunctionsAdapter.FUN> funcs)
        {
            points = new SurfacePoints3D(arg.PointsAmountX, arg.PointsAmountZ);
            InnerDataPoint3D inner = new InnerDataPoint3D();

            inner.Screen_Height = arg.Height;
            inner.Screen_Width = arg.Width;
            inner.X_Min = arg.dXBeg;
            inner.X_Max = arg.dXEnd;
            inner.Y_Min = arg.dYBeg;
            inner.Y_Max = arg.dYEnd;
            inner.Z_Min = arg.dZBeg;
            inner.Z_Max = arg.dZEnd;
            inner.CalculateScales();

            double x = arg.dXBeg;
            double z;
            double x_step;
            double z_step;
            Point3D point;
            double y;
            bool is_value;
            List<double> prevs = null;
            funcs = null;

            x_step = ((arg.dXEnd - arg.dXBeg) / (arg.PointsAmountX - 1));
            z_step = ((arg.dZEnd - arg.dZBeg) / (arg.PointsAmountZ - 1));

            for (int i = 0; i < points.X_AM; i++)
            {
                z = arg.dZBeg;
                for (int j = 0; j < points.Z_AM; j++)
                {
                    if (arg.Type == FunctionType.D)
                    {
                        is_value = false;
                        y = 0.0;
                    }
                    else
                    {
                        if (arg.Fun.CalculateFunction2DT(ref prevs, ref funcs, x, z, arg.Time, out y))
                        {
                            is_value = true;
                        }
                        else
                        {
                            is_value = false;
                        }
                    }
                    if (is_value)
                    {
                        point = new Point3D(inner);
                        point.X = x;
                        point.Z = z;
                        point.Y = y;
                        points[i, j] = point;
                        points.SetArguments(i, j, prevs);
                    }
                    else
                    {
                        point = null;
                        points[i, j] = point;
                    }
                    z += z_step;
                }
                x += x_step;
            }
            return true;
        }

        public static bool CalculateFunctionPoints(out MeshPoints3D mesh, ThreeDimFunctionArguments arg, out List<Functions.FunctionsAdapter.FUN> funcs)
        {
            MeshPoints4D mesh4D = new MeshPoints4D(arg.PointsAmountX, arg.PointsAmountZ, arg.PointsAmountY);
            InnerDataPoint3D inner = new InnerDataPoint3D();
            InnerDataPoint4D inner4D = new InnerDataPoint4D();

            inner.Screen_Height = arg.Height;
            inner.Screen_Width = arg.Width;
            inner.X_Min = arg.dXBeg;
            inner.X_Max = arg.dXEnd;
            inner.Y_Min = arg.dYBeg;
            inner.Y_Max = arg.dYEnd;
            inner.Z_Min = arg.dZBeg;
            inner.Z_Max = arg.dZEnd;
            inner.CalculateScales();

            inner4D.Q_Min = arg.QMin;
            inner4D.Q_Max = arg.QMax;

            double x = arg.dXBeg;
            double z = arg.dZBeg;
            double q;
            double x_step;
            double z_step;
            double y_step;
            Point4D point;
            double y = arg.dYBeg;
            bool is_value;
            
            List<double> prevs = null;
            funcs = null;
            
            x_step = ((arg.dXEnd - arg.dXBeg) / (arg.PointsAmountX - 1));
            z_step = ((arg.dZEnd - arg.dZBeg) / (arg.PointsAmountZ - 1));
            y_step = ((arg.dYEnd - arg.dYBeg) / (arg.PointsAmountY - 1));

            for (int i = 0; i < mesh4D.X_AM; i++)
            {
                z = arg.dZBeg;
                for (int j = 0; j < mesh4D.Z_AM; j++)
                {
                    y = arg.dYBeg;
                    for (int k = 0; k < mesh4D.Y_AM; k++)
                    {
                        if (arg.Type == FunctionType.D)
                        {
                            is_value = false;
                            q = 0.0;
                        }
                        else
                        {
                            if (arg.Fun.CalculateFunction3DT(ref prevs, ref funcs, x, z, y, arg.Time, out q))
                            {
                                is_value = true;
                            }
                            else
                            {
                                is_value = false;
                            }
                        }
                        if (is_value)
                        {
                            point = new Point4D(inner4D, inner);
                            point.X = x;
                            point.Z = z;
                            point.Y = y;
                            point.Q = q;
                            mesh4D.SetFunctionArguments(i, j, k, prevs);
                            mesh4D[i, j, k] = point;
                        }
                        else
                        {
                            prevs = null;
                            point = null;
                            mesh4D[i, j, k] = point;
                        }
                        y += y_step;
                    }
                    z += z_step;
                }
                x += x_step;
            }

            mesh = MeshPoints3D.GetMesh3D(mesh4D);
            return true;
        }
    }
}
